@@ -20,10 +20,10 @@
 #include <mongocxx/client.hpp>
 #include <mongocxx/instance.hpp>
 
+using bsoncxx::type;
 using bsoncxx::builder::basic::kvp;
 using bsoncxx::builder::basic::make_array;
 using bsoncxx::builder::basic::make_document;
-using bsoncxx::type;
 
 // Document model, showing array with nested documents:
 //
@@ -75,8 +75,8 @@ void iterate_messagelist(const bsoncxx::document::element& ele) {
                 if (status && status.type() == type::k_int32) {
                     std::cout << "status: " << status.get_int32().value << std::endl;
                 }
-                if (msg && msg.type() == type::k_utf8) {
-                    std::cout << "msg: " << msg.get_utf8().value << std::endl;
+                if (msg && msg.type() == type::k_string) {
+                    std::cout << "msg: " << msg.get_string().value << std::endl;
                 }
             } else {
                 std::cout << "Message is not a document" << std::endl;
