@@ -12,12 +12,11 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <bsoncxx/array/element.hpp>
-
-#include <cstdlib>
-#include <cstring>
 #include <stdexcept>
 
+#include <bsoncxx/array/element.hpp>
+#include <bsoncxx/types/bson_value/view.hpp>
+
 #include <bsoncxx/config/private/prelude.hh>
 
 namespace bsoncxx {
@@ -32,6 +31,22 @@ element::element(const std::uint8_t* raw,
                  std::uint32_t keylen)
     : document::element(raw, length, offset, keylen) {}
 
-}  // namespace document
+bool BSONCXX_CALL operator==(const element& elem, const types::bson_value::view& v) {
+    return elem.get_value() == v;
+}
+
+bool BSONCXX_CALL operator==(const types::bson_value::view& v, const element& elem) {
+    return elem == v;
+}
+
+bool BSONCXX_CALL operator!=(const element& elem, const types::bson_value::view& v) {
+    return !(elem == v);
+}
+
+bool BSONCXX_CALL operator!=(const types::bson_value::view& v, const element& elem) {
+    return !(elem == v);
+}
+
+}  // namespace array
 BSONCXX_INLINE_NAMESPACE_END
 }  // namespace bsoncxx
