@@ -13,6 +13,13 @@
 // limitations under the License.
 
 #pragma once
+#if defined(__clang__)
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wfloat-equal"
+#elif defined(__GNUC__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wfloat-equal"
+#endif
 
 #include <chrono>
 #include <cstring>
@@ -32,7 +39,7 @@ BSONCXX_INLINE_NAMESPACE_BEGIN
 /// An enumeration of each BSON type.
 /// These x-macros will expand to be of the form:
 ///    k_double = 0x01,
-///    k_utf8 = 0x02,
+///    k_string = 0x02,
 ///    k_document = 0x03,
 ///    k_array = 0x04 ...
 ///
@@ -40,6 +47,7 @@ enum class type : std::uint8_t {
 #define BSONCXX_ENUM(name, val) k_##name = val,
 #include <bsoncxx/enums/type.hpp>
 #undef BSONCXX_ENUM
+    k_utf8 = 0x02,
 };
 
 ///
@@ -52,6 +60,7 @@ enum class type : std::uint8_t {
 ///   k_uuid = 0x04,
 ///   k_md5 = 0x05,
 ///   k_encrypted = 0x06,
+///   k_column = 0x07,
 ///   k_user = 0x80
 ///
 enum class binary_sub_type : std::uint8_t {
@@ -110,19 +119,19 @@ BSONCXX_INLINE bool operator==(const b_double& lhs, const b_double& rhs) {
 ///
 /// A BSON UTF-8 encoded string value.
 ///
-struct BSONCXX_API b_utf8 {
-    static constexpr auto type_id = type::k_utf8;
+struct BSONCXX_API b_string {
+    static constexpr auto type_id = type::k_string;
 
     ///
-    /// Constructor for b_utf8.
+    /// Constructor for b_string.
     ///
-    /// @param value
+    /// @param t
     ///   The value to wrap.
     ///
     template <typename T,
-              typename std::enable_if<!std::is_same<b_utf8, typename std::decay<T>::type>::value,
+              typename std::enable_if<!std::is_same<b_string, typename std::decay<T>::type>::value,
                                       int>::type = 0>
-    BSONCXX_INLINE explicit b_utf8(T&& t) : value(std::forward<T>(t)) {}
+    BSONCXX_INLINE explicit b_string(T&& t) : value(std::forward<T>(t)) {}
 
     stdx::string_view value;
 
@@ -135,14 +144,21 @@ struct BSONCXX_API b_utf8 {
 };
 
 ///
-/// free function comparator for b_utf8
+/// free function comparator for b_string
 ///
-/// @relatesalso b_utf8
+/// @relatesalso b_string
 ///
-BSONCXX_INLINE bool operator==(const b_utf8& lhs, const b_utf8& rhs) {
+BSONCXX_INLINE bool operator==(const b_string& lhs, const b_string& rhs) {
     return lhs.value == rhs.value;
 }
 
+///
+/// This class has been renamed to b_string
+///
+/// @deprecated use b_string instead.
+///
+BSONCXX_DEPRECATED typedef b_string b_utf8;
+
 ///
 /// A BSON document value.
 ///
@@ -218,7 +234,7 @@ struct BSONCXX_API b_binary {
 ///
 BSONCXX_INLINE bool operator==(const b_binary& lhs, const b_binary& rhs) {
     return lhs.sub_type == rhs.sub_type && lhs.size == rhs.size &&
-           (std::memcmp(lhs.bytes, rhs.bytes, lhs.size) == 0);
+           (!lhs.size || (std::memcmp(lhs.bytes, rhs.bytes, lhs.size) == 0));
 }
 
 ///
@@ -301,7 +317,7 @@ struct BSONCXX_API b_date {
     ///
     /// Constructor for b_date
     ///
-    /// @param value
+    /// @param tp
     ///   A system_clock time_point.
     ///
     BSONCXX_INLINE
@@ -424,7 +440,7 @@ struct BSONCXX_API b_code {
     ///
     /// Constructor for b_code.
     ///
-    /// @param code
+    /// @param t
     ///   The js code
     ///
     template <typename T,
@@ -463,7 +479,7 @@ struct BSONCXX_API b_symbol {
     ///
     /// Constructor for b_symbol.
     ///
-    /// @param symbol
+    /// @param t
     ///   The symbol.
     ///
     template <typename T,
@@ -610,7 +626,7 @@ struct BSONCXX_API b_decimal128 {
     ///
     /// Constructor for b_decimal128.
     ///
-    /// @param value
+    /// @param t
     ///   The value to wrap.
     ///
     template <
@@ -673,3 +689,9 @@ BSONCXX_INLINE_NAMESPACE_END
 }  // namespace bsoncxx
 
 #include <bsoncxx/config/postlude.hpp>
+
+#if defined(__clang__)
+#pragma clang diagnostic pop
+#elif defined(__GNUC__)
+#pragma GCC diagnostic pop
+#endif
