@@ -12,8 +12,6 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <mongocxx/collection.hpp>
-
 #include <chrono>
 #include <cstdint>
 #include <limits>
@@ -33,6 +31,7 @@
 #include <bsoncxx/types.hpp>
 #include <mongocxx/bulk_write.hpp>
 #include <mongocxx/client.hpp>
+#include <mongocxx/collection.hpp>
 #include <mongocxx/exception/error_code.hpp>
 #include <mongocxx/exception/logic_error.hpp>
 #include <mongocxx/exception/operation_exception.hpp>
@@ -60,12 +59,12 @@
 
 #include <mongocxx/config/private/prelude.hh>
 
+using bsoncxx::builder::concatenate;
 using bsoncxx::builder::basic::kvp;
 using bsoncxx::builder::basic::make_array;
 using bsoncxx::builder::basic::make_document;
 using bsoncxx::builder::basic::sub_array;
 using bsoncxx::builder::basic::sub_document;
-using bsoncxx::builder::concatenate;
 using bsoncxx::document::view_or_value;
 
 namespace {
@@ -90,7 +89,6 @@ mongocxx::stdx::optional<bsoncxx::document::value> find_and_modify(
     mongoc_find_and_modify_flags_t flags,
     bool bypass,
     mongocxx::stdx::optional<bsoncxx::array::view_or_value> array_filters,
-    const mongocxx::stdx::optional<mongocxx::hint>& hint,
     const T& options) {
     using unique_opts =
         std::unique_ptr<mongoc_find_and_modify_opts_t,
@@ -102,11 +100,11 @@ mongocxx::stdx::optional<bsoncxx::document::value> find_and_modify(
     ::bson_error_t error;
 
     // Write concern, collation, and session are passed in "extra".
-    if (options.write_concern()) {
-        if (!options.write_concern()->is_acknowledged() && options.collation()) {
+    if (const auto& wc = options.write_concern()) {
+        if (!wc->is_acknowledged() && options.collation()) {
             throw mongocxx::logic_error{mongocxx::error_code::k_invalid_parameter};
         }
-        extra.append(kvp("writeConcern", options.write_concern()->to_document()));
+        extra.append(kvp("writeConcern", wc->to_document()));
     }
 
     if (session_t) {
@@ -121,18 +119,26 @@ mongocxx::stdx::optional<bsoncxx::document::value> find_and_modify(
         extra.append(concatenate(session_id.view()));
     }
 
-    if (options.collation()) {
-        extra.append(kvp("collation", *options.collation()));
+    if (const auto& collation = options.collation()) {
+        extra.append(kvp("collation", *collation));
     }
 
     if (array_filters) {
         extra.append(kvp("arrayFilters", *array_filters));
     }
 
-    if (hint) {
+    if (const auto& hint = options.hint()) {
         extra.append(kvp("hint", hint->to_value()));
     }
 
+    if (const auto& let = options.let()) {
+        extra.append(kvp("let", *let));
+    }
+
+    if (const auto& comment = options.comment()) {
+        extra.append(kvp("comment", *comment));
+    }
+
     scoped_bson_t extra_bson{extra.view()};
     mongocxx::libmongoc::find_and_modify_opts_append(opts.get(), extra_bson.bson());
 
@@ -145,19 +151,19 @@ mongocxx::stdx::optional<bsoncxx::document::value> find_and_modify(
         mongocxx::libmongoc::find_and_modify_opts_set_bypass_document_validation(opts.get(), true);
     }
 
-    if (options.sort()) {
-        scoped_bson_t sort_bson{*options.sort()};
+    if (const auto& sort = options.sort()) {
+        scoped_bson_t sort_bson{*sort};
         mongocxx::libmongoc::find_and_modify_opts_set_sort(opts.get(), sort_bson.bson());
     }
 
-    if (options.projection()) {
-        scoped_bson_t projection_bson{*options.projection()};
+    if (const auto& projection = options.projection()) {
+        scoped_bson_t projection_bson{*projection};
         mongocxx::libmongoc::find_and_modify_opts_set_fields(opts.get(), projection_bson.bson());
     }
 
-    if (options.max_time()) {
+    if (const auto& max_time = options.max_time()) {
         mongocxx::libmongoc::find_and_modify_opts_set_max_time_ms(
-            opts.get(), static_cast<uint32_t>(options.max_time()->count()));
+            opts.get(), static_cast<uint32_t>(max_time->count()));
     }
 
     // Upsert, remove, and new are passed in flags.
@@ -179,7 +185,9 @@ mongocxx::stdx::optional<bsoncxx::document::value> find_and_modify(
 
     bsoncxx::document::view reply_view = reply.view();
 
-    if (reply_view["value"].type() == bsoncxx::type::k_null) {
+    const auto value = reply_view["value"];
+
+    if (!value || value.type() == bsoncxx::type::k_null) {
         return mongocxx::stdx::optional<bsoncxx::document::value>{};
     }
 
@@ -300,82 +308,87 @@ namespace {
 bsoncxx::builder::basic::document build_find_options_document(const options::find& options) {
     bsoncxx::builder::basic::document options_builder;
 
-    if (options.allow_disk_use()) {
-        options_builder.append(kvp("allowDiskUse", *options.allow_disk_use()));
+    if (const auto& adu = options.allow_disk_use()) {
+        options_builder.append(kvp("allowDiskUse", *adu));
     }
 
-    if (options.allow_partial_results()) {
-        options_builder.append(kvp("allowPartialResults", *options.allow_partial_results()));
+    if (const auto& apr = options.allow_partial_results()) {
+        options_builder.append(kvp("allowPartialResults", *apr));
     }
 
-    if (options.batch_size()) {
-        options_builder.append(kvp("batchSize", *options.batch_size()));
+    if (const auto& batch_size = options.batch_size()) {
+        options_builder.append(kvp("batchSize", *batch_size));
     }
 
-    if (options.collation()) {
-        options_builder.append(kvp("collation", *options.collation()));
+    if (const auto& collation = options.collation()) {
+        options_builder.append(kvp("collation", *collation));
     }
 
-    if (options.comment()) {
-        options_builder.append(kvp("comment", *options.comment()));
+    // Prioritize new comment option over old $comment modifier.
+    if (const auto& comment = options.comment_option()) {
+        options_builder.append(kvp("comment", *comment));
+    } else if (const auto& comment = options.comment()) {
+        options_builder.append(kvp("comment", *comment));
     }
 
-    if (options.cursor_type()) {
-        if (*options.cursor_type() == cursor::type::k_tailable) {
+    if (const auto& cursor_type = options.cursor_type()) {
+        if (*cursor_type == cursor::type::k_tailable) {
             options_builder.append(kvp("tailable", bsoncxx::types::b_bool{true}));
-        } else if (*options.cursor_type() == cursor::type::k_tailable_await) {
+        } else if (*cursor_type == cursor::type::k_tailable_await) {
             options_builder.append(kvp("tailable", bsoncxx::types::b_bool{true}));
             options_builder.append(kvp("awaitData", bsoncxx::types::b_bool{true}));
-        } else if (*options.cursor_type() == cursor::type::k_non_tailable) {
+        } else if (*cursor_type == cursor::type::k_non_tailable) {
         } else {
             throw logic_error{error_code::k_invalid_parameter};
         }
     }
 
-    if (options.hint()) {
-        options_builder.append(kvp("hint", options.hint()->to_value()));
+    if (const auto& hint = options.hint()) {
+        options_builder.append(kvp("hint", hint->to_value()));
     }
 
-    if (options.limit()) {
-        options_builder.append(kvp("limit", *options.limit()));
+    if (const auto& let = options.let()) {
+        options_builder.append(kvp("let", *let));
     }
 
-    if (options.max()) {
-        options_builder.append(kvp("max", *options.max()));
+    if (const auto& limit = options.limit()) {
+        options_builder.append(kvp("limit", *limit));
     }
 
-    if (options.max_time()) {
-        options_builder.append(
-            kvp("maxTimeMS", bsoncxx::types::b_int64{options.max_time()->count()}));
+    if (const auto& max = options.max()) {
+        options_builder.append(kvp("max", *max));
     }
 
-    if (options.min()) {
-        options_builder.append(kvp("min", *options.min()));
+    if (const auto& max_time = options.max_time()) {
+        options_builder.append(kvp("maxTimeMS", bsoncxx::types::b_int64{max_time->count()}));
     }
 
-    if (options.no_cursor_timeout()) {
-        options_builder.append(kvp("noCursorTimeout", *options.no_cursor_timeout()));
+    if (const auto& min = options.min()) {
+        options_builder.append(kvp("min", *min));
     }
 
-    if (options.projection()) {
-        options_builder.append(
-            kvp("projection", bsoncxx::types::b_document{*options.projection()}));
+    if (const auto& nct = options.no_cursor_timeout()) {
+        options_builder.append(kvp("noCursorTimeout", *nct));
     }
 
-    if (options.return_key()) {
-        options_builder.append(kvp("returnKey", *options.return_key()));
+    if (const auto& projection = options.projection()) {
+        options_builder.append(kvp("projection", bsoncxx::types::b_document{*projection}));
     }
 
-    if (options.show_record_id()) {
-        options_builder.append(kvp("showRecordId", *options.show_record_id()));
+    if (const auto& return_key = options.return_key()) {
+        options_builder.append(kvp("returnKey", *return_key));
     }
 
-    if (options.skip()) {
-        options_builder.append(kvp("skip", *options.skip()));
+    if (const auto& show_record_id = options.show_record_id()) {
+        options_builder.append(kvp("showRecordId", *show_record_id));
     }
 
-    if (options.sort()) {
-        options_builder.append(kvp("sort", bsoncxx::types::b_document{*options.sort()}));
+    if (const auto& skip = options.skip()) {
+        options_builder.append(kvp("skip", *skip));
+    }
+
+    if (const auto& sort = options.sort()) {
+        options_builder.append(kvp("sort", bsoncxx::types::b_document{*sort}));
     }
 
     return options_builder;
@@ -502,12 +515,16 @@ stdx::optional<result::insert_one> collection::_insert_one(const client_session*
 
     options::bulk_write bulk_opts;
 
-    if (options.write_concern()) {
-        bulk_opts.write_concern(*options.write_concern());
+    if (const auto& wc = options.write_concern()) {
+        bulk_opts.write_concern(*wc);
     }
 
-    if (options.bypass_document_validation()) {
-        bulk_opts.bypass_document_validation(*options.bypass_document_validation());
+    if (const auto& bdv = options.bypass_document_validation()) {
+        bulk_opts.bypass_document_validation(*bdv);
+    }
+
+    if (const auto& comment = options.comment()) {
+        bulk_opts.comment(*comment);
     }
 
     class bulk_write bulk_op {
@@ -566,22 +583,34 @@ stdx::optional<result::replace_one> collection::_replace_one(const client_sessio
                                                              const options::replace& options) {
     options::bulk_write bulk_opts;
 
-    if (options.bypass_document_validation()) {
-        bulk_opts.bypass_document_validation(*options.bypass_document_validation());
+    if (const auto& bdv = options.bypass_document_validation()) {
+        bulk_opts.bypass_document_validation(*bdv);
     }
-    if (options.write_concern()) {
-        bulk_opts.write_concern(*options.write_concern());
+
+    if (const auto& wc = options.write_concern()) {
+        bulk_opts.write_concern(*wc);
+    }
+
+    if (const auto& let = options.let()) {
+        bulk_opts.let(*let);
+    }
+
+    if (const auto& comment = options.comment()) {
+        bulk_opts.comment(*comment);
     }
 
     model::replace_one replace_op(std::move(filter), std::move(replacement));
-    if (options.collation()) {
-        replace_op.collation(*options.collation());
+
+    if (const auto& collation = options.collation()) {
+        replace_op.collation(*collation);
     }
-    if (options.hint()) {
-        replace_op.hint(*options.hint());
+
+    if (const auto& hint = options.hint()) {
+        replace_op.hint(*hint);
     }
-    if (options.upsert()) {
-        replace_op.upsert(*options.upsert());
+
+    if (const auto& upsert = options.upsert()) {
+        replace_op.upsert(*upsert);
     }
 
     return _replace_one(session, bulk_opts, replace_op);
@@ -606,27 +635,40 @@ stdx::optional<result::update> collection::_update_many(const client_session* se
                                                         const options::update& options) {
     options::bulk_write bulk_opts;
 
-    if (options.bypass_document_validation()) {
-        bulk_opts.bypass_document_validation(*options.bypass_document_validation());
+    if (const auto& bdv = options.bypass_document_validation()) {
+        bulk_opts.bypass_document_validation(*bdv);
     }
-    if (options.write_concern()) {
-        bulk_opts.write_concern(*options.write_concern());
+
+    if (const auto& wc = options.write_concern()) {
+        bulk_opts.write_concern(*wc);
+    }
+
+    if (const auto& let = options.let()) {
+        bulk_opts.let(*let);
+    }
+
+    if (const auto& comment = options.comment()) {
+        bulk_opts.comment(*comment);
     }
 
     auto bulk_op = session ? create_bulk_write(*session, bulk_opts) : create_bulk_write(bulk_opts);
 
     model::update_many update_op(std::move(filter), std::move(update));
-    if (options.collation()) {
-        update_op.collation(*options.collation());
+
+    if (const auto& collation = options.collation()) {
+        update_op.collation(*collation);
     }
-    if (options.hint()) {
-        update_op.hint(*options.hint());
+
+    if (const auto& hint = options.hint()) {
+        update_op.hint(*hint);
     }
-    if (options.upsert()) {
-        update_op.upsert(*options.upsert());
+
+    if (const auto& upsert = options.upsert()) {
+        update_op.upsert(*upsert);
     }
-    if (options.array_filters()) {
-        update_op.array_filters(*options.array_filters());
+
+    if (const auto& array_filters = options.array_filters()) {
+        update_op.array_filters(*array_filters);
     }
 
     bulk_op.append(update_op);
@@ -686,27 +728,40 @@ stdx::optional<result::update> collection::_update_one(const client_session* ses
                                                        const options::update& options) {
     options::bulk_write bulk_opts;
 
-    if (options.bypass_document_validation()) {
-        bulk_opts.bypass_document_validation(*options.bypass_document_validation());
+    if (const auto& bdv = options.bypass_document_validation()) {
+        bulk_opts.bypass_document_validation(*bdv);
     }
-    if (options.write_concern()) {
-        bulk_opts.write_concern(*options.write_concern());
+
+    if (const auto& wc = options.write_concern()) {
+        bulk_opts.write_concern(*wc);
+    }
+
+    if (const auto& let = options.let()) {
+        bulk_opts.let(*let);
+    }
+
+    if (const auto& comment = options.comment()) {
+        bulk_opts.comment(*comment);
     }
 
     auto bulk_op = session ? create_bulk_write(*session, bulk_opts) : create_bulk_write(bulk_opts);
 
     model::update_one update_op(std::move(filter), std::move(update));
-    if (options.collation()) {
-        update_op.collation(*options.collation());
+
+    if (const auto& collation = options.collation()) {
+        update_op.collation(*collation);
     }
-    if (options.hint()) {
-        update_op.hint(*options.hint());
+
+    if (const auto& hint = options.hint()) {
+        update_op.hint(*hint);
     }
-    if (options.upsert()) {
-        update_op.upsert(*options.upsert());
+
+    if (const auto& upsert = options.upsert()) {
+        update_op.upsert(*upsert);
     }
-    if (options.array_filters()) {
-        update_op.array_filters(*options.array_filters());
+
+    if (const auto& array_filters = options.array_filters()) {
+        update_op.array_filters(*array_filters);
     }
 
     bulk_op.append(update_op);
@@ -764,19 +819,30 @@ stdx::optional<result::delete_result> collection::_delete_many(
     const client_session* session, view_or_value filter, const options::delete_options& options) {
     options::bulk_write bulk_opts;
 
-    if (options.write_concern()) {
-        bulk_opts.write_concern(*options.write_concern());
+    if (const auto& wc = options.write_concern()) {
+        bulk_opts.write_concern(*wc);
+    }
+
+    if (const auto& let = options.let()) {
+        bulk_opts.let(*let);
+    }
+
+    if (const auto& comment = options.comment()) {
+        bulk_opts.comment(*comment);
     }
 
     auto bulk_op = session ? create_bulk_write(*session, bulk_opts) : create_bulk_write(bulk_opts);
 
     model::delete_many delete_op(filter);
-    if (options.collation()) {
-        delete_op.collation(*options.collation());
+
+    if (const auto& collation = options.collation()) {
+        delete_op.collation(*collation);
     }
-    if (options.hint()) {
-        delete_op.hint(*options.hint());
+
+    if (const auto& hint = options.hint()) {
+        delete_op.hint(*hint);
     }
+
     bulk_op.append(delete_op);
 
     auto result = bulk_op.execute();
@@ -801,19 +867,30 @@ stdx::optional<result::delete_result> collection::_delete_one(
     const client_session* session, view_or_value filter, const options::delete_options& options) {
     options::bulk_write bulk_opts;
 
-    if (options.write_concern()) {
-        bulk_opts.write_concern(*options.write_concern());
+    if (const auto& wc = options.write_concern()) {
+        bulk_opts.write_concern(*wc);
+    }
+
+    if (const auto& let = options.let()) {
+        bulk_opts.let(*let);
+    }
+
+    if (const auto& comment = options.comment()) {
+        bulk_opts.comment(*comment);
     }
 
     auto bulk_op = session ? create_bulk_write(*session, bulk_opts) : create_bulk_write(bulk_opts);
 
     model::delete_one delete_op(filter);
-    if (options.collation()) {
-        delete_op.collation(*options.collation());
+
+    if (const auto collation = options.collation()) {
+        delete_op.collation(*collation);
     }
-    if (options.hint()) {
-        delete_op.hint(*options.hint());
+
+    if (const auto hint = options.hint()) {
+        delete_op.hint(*hint);
     }
+
     bulk_op.append(delete_op);
 
     auto result = bulk_op.execute();
@@ -855,7 +932,6 @@ stdx::optional<bsoncxx::document::value> collection::_find_one_and_replace(
                            flags,
                            options.bypass_document_validation().value_or(false),
                            stdx::nullopt,
-                           options.hint(),
                            options);
 }
 
@@ -893,7 +969,6 @@ stdx::optional<bsoncxx::document::value> collection::_find_one_and_update(
                            flags,
                            options.bypass_document_validation().value_or(false),
                            options.array_filters(),
-                           options.hint(),
                            options);
 }
 
@@ -951,7 +1026,6 @@ stdx::optional<bsoncxx::document::value> collection::_find_one_and_delete(
                            MONGOC_FIND_AND_MODIFY_REMOVE,
                            false,
                            stdx::nullopt,
-                           options.hint(),
                            options);
 }
 
@@ -975,34 +1049,38 @@ std::int64_t collection::_count_documents(const client_session* session,
     bson_error_t error;
     const mongoc_read_prefs_t* read_prefs = NULL;
 
-    if (options.read_preference()) {
-        read_prefs = options.read_preference()->_impl->read_preference_t;
+    if (const auto& rp = options.read_preference()) {
+        read_prefs = rp->_impl->read_preference_t;
     }
 
     bsoncxx::builder::basic::document opts_builder;
 
-    if (options.collation()) {
-        opts_builder.append(kvp("collation", *options.collation()));
+    if (const auto& collation = options.collation()) {
+        opts_builder.append(kvp("collation", *collation));
     }
 
-    if (options.max_time()) {
-        opts_builder.append(kvp("maxTimeMS", bsoncxx::types::b_int64{options.max_time()->count()}));
+    if (const auto& max_time = options.max_time()) {
+        opts_builder.append(kvp("maxTimeMS", bsoncxx::types::b_int64{max_time->count()}));
     }
 
-    if (options.hint()) {
-        opts_builder.append(kvp("hint", options.hint()->to_value()));
+    if (const auto& hint = options.hint()) {
+        opts_builder.append(kvp("hint", hint->to_value()));
+    }
+
+    if (const auto& comment = options.comment()) {
+        opts_builder.append(kvp("comment", *comment));
     }
 
     if (session) {
         opts_builder.append(bsoncxx::builder::concatenate_doc{session->_get_impl().to_document()});
     }
 
-    if (options.skip()) {
-        opts_builder.append(kvp("skip", *options.skip()));
+    if (const auto& skip = options.skip()) {
+        opts_builder.append(kvp("skip", *skip));
     }
 
-    if (options.limit()) {
-        opts_builder.append(kvp("limit", *options.limit()));
+    if (const auto& limit = options.limit()) {
+        opts_builder.append(kvp("limit", *limit));
     }
 
     scoped_bson_t opts_bson{opts_builder.view()};
@@ -1035,14 +1113,18 @@ std::int64_t collection::estimated_document_count(
 
     const mongoc_read_prefs_t* read_prefs = NULL;
 
-    if (options.read_preference()) {
-        read_prefs = options.read_preference()->_impl->read_preference_t;
+    if (const auto& rp = options.read_preference()) {
+        read_prefs = rp->_impl->read_preference_t;
     }
 
     bsoncxx::builder::basic::document opts_builder;
 
-    if (options.max_time()) {
-        opts_builder.append(kvp("maxTimeMS", bsoncxx::types::b_int64{options.max_time()->count()}));
+    if (const auto& max_time = options.max_time()) {
+        opts_builder.append(kvp("maxTimeMS", bsoncxx::types::b_int64{max_time->count()}));
+    }
+
+    if (const auto& comment = options.comment()) {
+        opts_builder.append(kvp("comment", *comment));
     }
 
     scoped_bson_t opts_bson{opts_builder.view()};
@@ -1095,14 +1177,17 @@ cursor collection::_distinct(const client_session* session,
                            kvp("key", field_name.view()),
                            kvp("query", bsoncxx::types::b_document{query}));
 
-    if (options.max_time()) {
-        command_builder.append(
-            kvp("maxTimeMS", bsoncxx::types::b_int64{options.max_time()->count()}));
+    if (const auto& max_time = options.max_time()) {
+        command_builder.append(kvp("maxTimeMS", bsoncxx::types::b_int64{max_time->count()}));
     }
 
     bsoncxx::builder::basic::document opts_builder{};
-    if (options.collation()) {
-        opts_builder.append(kvp("collation", *options.collation()));
+    if (const auto& collation = options.collation()) {
+        opts_builder.append(kvp("collation", *collation));
+    }
+
+    if (const auto& comment = options.comment()) {
+        opts_builder.append(kvp("comment", *comment));
     }
 
     if (session) {
@@ -1110,8 +1195,8 @@ cursor collection::_distinct(const client_session* session,
     }
 
     const mongoc_read_prefs_t* rp_ptr = NULL;
-    if (options.read_preference()) {
-        rp_ptr = options.read_preference()->_impl->read_preference_t;
+    if (const auto& rp = options.read_preference()) {
+        rp_ptr = rp->_impl->read_preference_t;
     }
 
     //
@@ -1150,10 +1235,18 @@ cursor collection::_distinct(const client_session* session,
         throw bsoncxx::exception{bsoncxx::error_code::k_internal_error};
     }
 
+    const bson_t* error_document;
+
     cursor fake_cursor{libmongoc::cursor_new_from_command_reply_with_opts(
         _get_impl().client_impl->client_t, reply_bson, nullptr)};
-    if (libmongoc::cursor_error(fake_cursor._impl->cursor_t, &error)) {
-        throw_exception<operation_exception>(error);
+    if (libmongoc::cursor_error_document(fake_cursor._impl->cursor_t, &error, &error_document)) {
+        if (error_document) {
+            bsoncxx::document::value error_doc{
+                bsoncxx::document::view{bson_get_data(error_document), error_document->len}};
+            throw_exception<query_exception>(error_doc, error);
+        } else {
+            throw_exception<query_exception>(error);
+        }
     }
 
     return fake_cursor;
@@ -1204,9 +1297,8 @@ void collection::_drop(const client_session* session,
     // Throw an exception if the command failed, unless the failure was due to a non-existent
     // collection. We check for this failure using 'code', but we fall back to checking 'message'
     // for old server versions (3.0 and earlier) that do not send a code with the command response.
-    if (!result &&
-        !(error.code == ::MONGOC_ERROR_COLLECTION_DOES_NOT_EXIST ||
-          stdx::string_view{error.message} == stdx::string_view{"ns not found"})) {
+    if (!result && !(error.code == ::MONGOC_ERROR_COLLECTION_DOES_NOT_EXIST ||
+                     stdx::string_view{error.message} == stdx::string_view{"ns not found"})) {
         throw_exception<operation_exception>(error);
     }
 }
@@ -1296,13 +1388,21 @@ class index_view collection::indexes() {
 class bulk_write collection::_init_insert_many(const options::insert& options,
                                                const client_session* session) {
     options::bulk_write bulk_write_options;
+
     bulk_write_options.ordered(options.ordered().value_or(true));
-    if (options.write_concern()) {
-        bulk_write_options.write_concern(*options.write_concern());
+
+    if (const auto& wc = options.write_concern()) {
+        bulk_write_options.write_concern(*wc);
     }
-    if (options.bypass_document_validation()) {
-        bulk_write_options.bypass_document_validation(*options.bypass_document_validation());
+
+    if (const auto& bdv = options.bypass_document_validation()) {
+        bulk_write_options.bypass_document_validation(*bdv);
     }
+
+    if (const auto& comment = options.comment()) {
+        bulk_write_options.comment(*comment);
+    }
+
     if (session) {
         return create_bulk_write(*session, bulk_write_options);
     }
