@@ -12,15 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <mongocxx/gridfs/uploader.hpp>
-
 #include <algorithm>
 #include <chrono>
 #include <cstring>
-#include <iomanip>
-#include <ios>
 #include <limits>
-#include <sstream>
 
 #include <bsoncxx/builder/basic/document.hpp>
 #include <bsoncxx/builder/basic/kvp.hpp>
@@ -29,6 +24,7 @@
 #include <mongocxx/exception/gridfs_exception.hpp>
 #include <mongocxx/exception/logic_error.hpp>
 #include <mongocxx/gridfs/private/uploader.hh>
+#include <mongocxx/gridfs/uploader.hpp>
 
 #include <mongocxx/config/private/prelude.hh>
 
