@@ -48,7 +48,7 @@ class MONGOCXX_API auto_encryption {
     /// The given key vault client MUST outlive any client that has
     /// been enabled to use it through these options.
     ///
-    /// @param
+    /// @param client
     ///   A client to use for routing queries to the key vault collection.
     ///
     /// @return
@@ -78,7 +78,7 @@ class MONGOCXX_API auto_encryption {
     ///
     /// May only be set when enabling automatic encryption on a pool.
     ///
-    /// @param
+    /// @param pool
     ///   A pool to use for routing queries to the key vault collection.
     ///
     /// @return
@@ -127,8 +127,8 @@ class MONGOCXX_API auto_encryption {
     ///
     /// Sets the KMS providers to use for client side encryption.
     ///
-    /// Multiple KMS providers may be specified. Two KMS providers are
-    /// supported: "aws" and "local". The kmsProviders map values differ
+    /// Multiple KMS providers may be specified. The following KMS providers are
+    /// supported: "aws", "azure", "gcp", "kmip", and "local". The kmsProviders map values differ
     /// by provider:
     ///
     ///    aws: {
@@ -136,6 +136,23 @@ class MONGOCXX_API auto_encryption {
     ///      secretAccessKey: String
     ///    }
     ///
+    ///    azure: {
+    ///       tenantId: String,
+    ///       clientId: String,
+    ///       clientSecret: String,
+    ///       identityPlatformEndpoint: Optional<String> // Defaults to login.microsoftonline.com
+    ///    }
+    ///
+    ///    gcp: {
+    ///       email: String,
+    ///       privateKey: byte[] or String, // May be passed as a base64 encoded string.
+    ///       endpoint: Optional<String> // Defaults to oauth2.googleapis.com
+    ///    }
+    ///
+    ///    kmip: {
+    ///       endpoint: String
+    ///    }
+    ///
     ///    local: {
     ///      key: byte[96] // The master key used to encrypt/decrypt data keys.
     ///    }
@@ -158,6 +175,36 @@ class MONGOCXX_API auto_encryption {
     ///
     const stdx::optional<bsoncxx::document::view_or_value>& kms_providers() const;
 
+    ///
+    /// Sets the TLS options to use for client side encryption with a given KMS provider.
+    ///
+    /// Multiple KMS providers may be specified. Supported KMS providers are "aws", "azure", "gcp",
+    /// and "kmip". The map value has the same form for all supported providers:
+    ///
+    ///    <KMS provider name>: {
+    ///        tlsCaFile: Optional<String>
+    ///        tlsCertificateKeyFile: Optional<String>
+    ///        tlsCertificateKeyFilePassword: Optional<String>
+    ///    }
+    ///
+    /// @param tls_opts
+    ///   A document containing the TLS options.
+    ///
+    /// @return
+    ///   A reference to this object to facilitate method chaining.
+    ///
+    /// @see https://docs.mongodb.com/manual/core/security-client-side-encryption/
+    ///
+    auto_encryption& tls_opts(bsoncxx::document::view_or_value tls_opts);
+
+    ///
+    /// Gets the TLS options.
+    ///
+    /// @return
+    ///   An optional document containing the TLS options.
+    ///
+    const stdx::optional<bsoncxx::document::view_or_value>& tls_opts() const;
+
     ///
     /// Sets a local JSON schema.
     ///
@@ -255,6 +302,7 @@ class MONGOCXX_API auto_encryption {
     stdx::optional<mongocxx::pool*> _key_vault_pool;
     stdx::optional<ns_pair> _key_vault_namespace;
     stdx::optional<bsoncxx::document::view_or_value> _kms_providers;
+    stdx::optional<bsoncxx::document::view_or_value> _tls_opts;
     stdx::optional<bsoncxx::document::view_or_value> _schema_map;
     stdx::optional<bsoncxx::document::view_or_value> _extra_options;
 };
