@@ -20,6 +20,7 @@
 #include <bsoncxx/document/view_or_value.hpp>
 #include <bsoncxx/stdx/optional.hpp>
 #include <bsoncxx/string/view_or_value.hpp>
+#include <bsoncxx/types/bson_value/view_or_value.hpp>
 #include <mongocxx/cursor.hpp>
 #include <mongocxx/hint.hpp>
 #include <mongocxx/read_preference.hpp>
@@ -74,7 +75,7 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& allow_partial_results(bool allow_partial);
 
@@ -83,7 +84,7 @@ class MONGOCXX_API find {
     ///
     /// @return Whether partial results from mongos are allowed.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<bool>& allow_partial_results() const;
 
@@ -97,7 +98,7 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& batch_size(std::int32_t batch_size);
 
@@ -106,7 +107,7 @@ class MONGOCXX_API find {
     ///
     /// @return The current batch size.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<std::int32_t>& batch_size() const;
 
@@ -120,7 +121,7 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& collation(bsoncxx::document::view_or_value collation);
 
@@ -130,7 +131,7 @@ class MONGOCXX_API find {
     /// @return
     ///   The current collation.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<bsoncxx::document::view_or_value>& collation() const;
 
@@ -138,6 +139,8 @@ class MONGOCXX_API find {
     /// Attaches a comment to the query. If $comment also exists in the modifiers document then
     /// the comment field overwrites $comment.
     ///
+    /// @deprecated use comment_option instead.
+    ///
     /// @param comment
     ///   The comment to attach to this query.
     ///
@@ -145,16 +148,18 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& comment(bsoncxx::string::view_or_value comment);
 
     ///
     /// Gets the current comment attached to this query.
     ///
+    /// @deprecated use comment_option instead.
+    ///
     /// @return The comment attached to this query.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<bsoncxx::string::view_or_value>& comment() const;
 
@@ -168,7 +173,7 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& cursor_type(cursor::type cursor_type);
 
@@ -177,14 +182,14 @@ class MONGOCXX_API find {
     ///
     /// @return The current cursor type.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<cursor::type>& cursor_type() const;
 
     ///
     /// Sets the index to use for this operation.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     /// @note if the server already has a cached shape for this query, it may
     /// ignore a hint.
@@ -203,10 +208,62 @@ class MONGOCXX_API find {
     ///
     /// @return The current hint, if one is set.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<class hint>& hint() const;
 
+    ///
+    /// Set the value of the let option.
+    ///
+    /// @param let
+    ///   The new let option.
+    ///
+    /// @return
+    ///   A reference to the object on which this member function is being called.  This facilitates
+    ///   method chaining.
+    ///
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
+    ///
+    find& let(bsoncxx::document::view_or_value let);
+
+    ///
+    /// Gets the current value of the let option.
+    ///
+    /// @return
+    ///  The current let option.
+    ///
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
+    ///
+    const stdx::optional<bsoncxx::document::view_or_value> let() const;
+
+    ///
+    /// Set the value of the comment option.
+    ///
+    /// @note Not to be confused with the $comment query modifier.
+    ///
+    /// @param comment
+    ///   The new comment option.
+    ///
+    /// @return
+    ///   A reference to the object on which this member function is being called.  This facilitates
+    ///   method chaining.
+    ///
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
+    ///
+    find& comment_option(bsoncxx::types::bson_value::view_or_value comment);
+
+    ///
+    /// Gets the current value of the comment option.
+    ///
+    /// @note Not to be confused with the $comment query modifier.
+    ///
+    /// @return
+    ///  The current comment option.
+    ///
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
+    ///
+    const stdx::optional<bsoncxx::types::bson_value::view_or_value>& comment_option() const;
+
     ///
     /// Sets maximum number of documents to return.
     ///
@@ -216,7 +273,7 @@ class MONGOCXX_API find {
     /// @return
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& limit(std::int64_t limit);
 
@@ -225,7 +282,7 @@ class MONGOCXX_API find {
     ///
     /// @return The current limit.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<std::int64_t>& limit() const;
 
@@ -239,7 +296,7 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& max(bsoncxx::document::view_or_value max);
 
@@ -248,7 +305,7 @@ class MONGOCXX_API find {
     ///
     /// @return The exclusive upper bound for a specific index.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<bsoncxx::document::view_or_value>& max() const;
 
@@ -267,7 +324,7 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& max_await_time(std::chrono::milliseconds max_await_time);
 
@@ -277,7 +334,7 @@ class MONGOCXX_API find {
     ///
     /// @return The current max await time (in milliseconds).
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<std::chrono::milliseconds>& max_await_time() const;
 
@@ -291,7 +348,7 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& max_time(std::chrono::milliseconds max_time);
 
@@ -300,7 +357,7 @@ class MONGOCXX_API find {
     ///
     /// @return The current max time (in milliseconds).
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<std::chrono::milliseconds>& max_time() const;
 
@@ -314,7 +371,7 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& min(bsoncxx::document::view_or_value min);
 
@@ -323,7 +380,7 @@ class MONGOCXX_API find {
     ///
     /// @return The inclusive lower bound for a specific index.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<bsoncxx::document::view_or_value>& min() const;
 
@@ -338,7 +395,7 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& no_cursor_timeout(bool no_cursor_timeout);
 
@@ -347,7 +404,7 @@ class MONGOCXX_API find {
     ///
     /// @return The current no_cursor_timeout setting.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<bool>& no_cursor_timeout() const;
 
@@ -361,7 +418,7 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& projection(bsoncxx::document::view_or_value projection);
 
@@ -370,7 +427,7 @@ class MONGOCXX_API find {
     ///
     /// @return The current projection.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<bsoncxx::document::view_or_value>& projection() const;
 
@@ -384,7 +441,7 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& read_preference(class read_preference rp);
 
@@ -394,7 +451,7 @@ class MONGOCXX_API find {
     /// @return
     ///   The current read_preference.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<class read_preference>& read_preference() const;
 
@@ -410,7 +467,7 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& return_key(bool return_key);
 
@@ -422,7 +479,7 @@ class MONGOCXX_API find {
     ///   Whether index keys associated with the query results are returned, instead of the actual
     ///   query results themselves.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<bool>& return_key() const;
 
@@ -436,7 +493,7 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& show_record_id(bool show_record_id);
 
@@ -447,7 +504,7 @@ class MONGOCXX_API find {
     /// @return
     ///   Whether the record identifier is included.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<bool>& show_record_id() const;
 
@@ -461,7 +518,7 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& skip(std::int64_t skip);
 
@@ -470,7 +527,7 @@ class MONGOCXX_API find {
     ///
     /// @return The number of documents to skip.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<std::int64_t>& skip() const;
 
@@ -485,7 +542,7 @@ class MONGOCXX_API find {
     ///   A reference to the object on which this member function is being called.  This facilitates
     ///   method chaining.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     find& sort(bsoncxx::document::view_or_value ordering);
 
@@ -494,7 +551,7 @@ class MONGOCXX_API find {
     ///
     /// @return The current sort ordering.
     ///
-    /// @see https://docs.mongodb.com/master/reference/command/find/
+    /// @see https://docs.mongodb.com/manual/reference/command/find/
     ///
     const stdx::optional<bsoncxx::document::view_or_value>& sort() const;
 
@@ -506,6 +563,8 @@ class MONGOCXX_API find {
     stdx::optional<bsoncxx::string::view_or_value> _comment;
     stdx::optional<cursor::type> _cursor_type;
     stdx::optional<class hint> _hint;
+    stdx::optional<bsoncxx::document::view_or_value> _let;
+    stdx::optional<bsoncxx::types::bson_value::view_or_value> _comment_option;
     stdx::optional<std::int64_t> _limit;
     stdx::optional<bsoncxx::document::view_or_value> _max;
     stdx::optional<std::chrono::milliseconds> _max_await_time;
