@@ -12,10 +12,9 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#include <bsoncxx/document/view_or_value.hpp>
 #include <mongocxx/options/replace.hpp>
 
-#include <bsoncxx/array/view_or_value.hpp>
-#include <bsoncxx/document/view_or_value.hpp>
 #include <mongocxx/config/private/prelude.hh>
 
 namespace mongocxx {
@@ -32,6 +31,16 @@ replace& replace::hint(class hint index_hint) {
     return *this;
 }
 
+replace& replace::let(bsoncxx::document::view_or_value let) {
+    _let = let;
+    return *this;
+}
+
+replace& replace::comment(bsoncxx::types::bson_value::view_or_value comment) {
+    _comment = std::move(comment);
+    return *this;
+}
+
 replace& replace::collation(bsoncxx::document::view_or_value collation) {
     _collation = std::move(collation);
     return *this;
@@ -51,6 +60,14 @@ const stdx::optional<class hint>& replace::hint() const {
     return _hint;
 }
 
+const stdx::optional<bsoncxx::document::view_or_value> replace::let() const {
+    return _let;
+}
+
+const stdx::optional<bsoncxx::types::bson_value::view_or_value> replace::comment() const {
+    return _comment;
+}
+
 const stdx::optional<bool>& replace::bypass_document_validation() const {
     return _bypass_document_validation;
 }
