@@ -49,14 +49,14 @@ class uri;
 /// a single document. Note that writes must be made with majority write concern in order for reads
 /// to be linearizable.
 ///
-/// @see https://docs.mongodb.com/master/reference/read-concern/
+/// @see https://docs.mongodb.com/manual/reference/read-concern/
 ///
 class MONGOCXX_API read_concern {
    public:
     ///
     /// A class to represent the read concern level.
     ///
-    /// @see https://docs.mongodb.com/master/reference/read-concern/#read-concern-levels
+    /// @see https://docs.mongodb.com/manual/reference/read-concern/#read-concern-levels
     ///
     enum class level {
         k_local,
@@ -157,7 +157,8 @@ class MONGOCXX_API read_concern {
     friend client;
     friend collection;
     friend database;
-    friend options::transaction;
+    /// \relates mongocxx::options::transaction
+    friend mongocxx::options::transaction;
     friend uri;
 
     ///
