@@ -26,7 +26,6 @@
 #include <mongocxx/private/libbson.hh>
 #include <mongocxx/test_util/client_helpers.hh>
 #include <mongocxx/write_concern.hpp>
-
 #include <third_party/catch/include/helpers.hpp>
 
 namespace {
@@ -89,7 +88,7 @@ const auto destroy_interpose = [](mongoc_change_stream_t*) -> void {};
 
 TEST_CASE("Change stream options") {
     instance::current();
-    client mongodb_client{uri{}};
+    client mongodb_client{uri{}, test_util::add_test_server_api()};
 
     if (!test_util::is_replica_set(mongodb_client)) {
         WARN("skip: change streams require replica set");
@@ -112,7 +111,7 @@ TEST_CASE("Change stream options") {
 
 TEST_CASE("Spec Prose Tests") {
     instance::current();
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
 
     if (!test_util::is_replica_set(client)) {
         WARN("skip: change streams require replica set");
@@ -124,7 +123,7 @@ TEST_CASE("Spec Prose Tests") {
     coll.drop();
 
     write_concern wc_majority;
-    wc_majority.majority(std::chrono::milliseconds(0));
+    wc_majority.majority(std::chrono::seconds(30));
 
     // As a sanity check, we implement the first prose test. The behavior tested
     // by the prose tests is implemented and tested by the C driver, so we won't
@@ -137,30 +136,45 @@ TEST_CASE("Spec Prose Tests") {
         auto cs = client.watch(std::move(opts));
 
         // With WC majority, insert some documents to listen for.
-        auto doc1 = make_document(kvp("_id", 1));
-        auto doc2 = make_document(kvp("_id", 2));
-        auto doc3 = make_document(kvp("_id", 3));
+        auto doc1 = make_document(kvp("a", 1));
+        auto doc2 = make_document(kvp("b", 2));
+        auto doc3 = make_document(kvp("c", 3));
 
         options::insert insert_opts{};
         insert_opts.write_concern(wc_majority);
 
-        coll.insert_one(doc1.view(), insert_opts);
-        coll.insert_one(doc2.view(), insert_opts);
-        coll.insert_one(doc3.view(), insert_opts);
+        {
+            auto res = coll.insert_one(doc1.view(), insert_opts);
+            REQUIRE(res);
+            REQUIRE(res->result().inserted_count() == 1);
+        }
+        {
+            auto res = coll.insert_one(doc2.view(), insert_opts);
+            REQUIRE(res);
+            REQUIRE(res->result().inserted_count() == 1);
+        }
+        {
+            auto res = coll.insert_one(doc3.view(), insert_opts);
+            REQUIRE(res);
+            REQUIRE(res->result().inserted_count() == 1);
+        }
 
         // For each read, check the resume token is updated. We should
         // be reading the postBatchResumeToken on each read, since our
         // batch size is 1.
         auto it = cs.begin();
+        REQUIRE(it != cs.end());
         REQUIRE(cs.get_resume_token());
         auto token1 = bsoncxx::document::value(*cs.get_resume_token());
 
         it++;
+        REQUIRE(it != cs.end());
         REQUIRE(cs.get_resume_token());
         auto token2 = bsoncxx::document::value(*cs.get_resume_token());
         REQUIRE(token1 != token2);
 
         it++;
+        REQUIRE(it != cs.end());
         REQUIRE(cs.get_resume_token());
         auto token3 = bsoncxx::document::value(*cs.get_resume_token());
         REQUIRE(token2 != token3);
@@ -169,6 +183,7 @@ TEST_CASE("Spec Prose Tests") {
         // When out of docs, check that the resume token is the same as the last doc.
         it++;
         REQUIRE(it == cs.end());
+        REQUIRE(cs.get_resume_token());
         REQUIRE(*cs.get_resume_token() == token3);
     }
 }
@@ -177,7 +192,7 @@ TEST_CASE("Mock streams and error-handling") {
     MOCK_CHANGE_STREAM
 
     instance::current();
-    client mongodb_client{uri{}};
+    client mongodb_client{uri{}, test_util::add_test_server_api()};
     options::change_stream options{};
     database db = mongodb_client["streams"];
     collection events = db["events"];
@@ -320,19 +335,20 @@ TEST_CASE("Mock streams and error-handling") {
             REQUIRE(opts["batchSize"].get_int32() == batch_size);
             REQUIRE(opts["maxAwaitTimeMS"].get_int64() == 4);
             REQUIRE(opts["collation"].get_document().view() == collation);
-            REQUIRE(opts["fullDocument"].get_utf8().value ==
+            REQUIRE(opts["fullDocument"].get_string().value ==
                     bsoncxx::stdx::string_view{full_document});
             REQUIRE(opts["resumeAfter"].get_document().view() == resume_after);
         };
 
-        collection_watch->interpose([&](
-            const mongoc_collection_t* coll, const bson_t* pipeline, const bson_t* opts) {
-            std::string name = mongoc_collection_get_name(const_cast<mongoc_collection_t*>(coll));
-            REQUIRE(name == "collection");
-            check_pipeline_and_opts(pipeline, opts);
-            collection_watch_called = true;
-            return nullptr;
-        });
+        collection_watch->interpose(
+            [&](const mongoc_collection_t* coll, const bson_t* pipeline, const bson_t* opts) {
+                std::string name =
+                    mongoc_collection_get_name(const_cast<mongoc_collection_t*>(coll));
+                REQUIRE(name == "collection");
+                check_pipeline_and_opts(pipeline, opts);
+                collection_watch_called = true;
+                return nullptr;
+            });
 
         database_watch->interpose(
             [&](const mongoc_database_t* db, const bson_t* pipeline, const bson_t* opts) {
@@ -365,7 +381,7 @@ TEST_CASE("Mock streams and error-handling") {
 // Put this before other tests which assume the collections already exists.
 TEST_CASE("Create streams.events and assert we can read a single event", "[min36]") {
     instance::current();
-    client mongodb_client{uri{}};
+    client mongodb_client{uri{}, test_util::add_test_server_api()};
     if (!test_util::is_replica_set(mongodb_client)) {
         WARN("skip: change streams require replica set");
         return;
@@ -386,7 +402,7 @@ TEST_CASE("Create streams.events and assert we can read a single event", "[min36
 
 TEST_CASE("Give an invalid pipeline", "[min36]") {
     instance::current();
-    client mongodb_client{uri{}};
+    client mongodb_client{uri{}, test_util::add_test_server_api()};
     if (!test_util::is_replica_set(mongodb_client)) {
         WARN("skip: change streams require replica set");
         return;
@@ -416,7 +432,7 @@ TEST_CASE("Give an invalid pipeline", "[min36]") {
 
 TEST_CASE("Documentation Examples", "[min36]") {
     instance::current();
-    client mongodb_client{uri{}};
+    client mongodb_client{uri{}, test_util::add_test_server_api()};
     if (!test_util::is_replica_set(mongodb_client)) {
         WARN("skip: change streams require replica set");
         return;
@@ -485,7 +501,7 @@ TEST_CASE("Documentation Examples", "[min36]") {
 
 TEST_CASE("Watch 2 collections", "[min36]") {
     instance::current();
-    client mongodb_client{uri{}};
+    client mongodb_client{uri{}, test_util::add_test_server_api()};
     if (!test_util::is_replica_set(mongodb_client)) {
         WARN("skip: change streams require replica set");
         return;
@@ -533,7 +549,7 @@ TEST_CASE("Watch 2 collections", "[min36]") {
 
 TEST_CASE("Watch a Collection", "[min36]") {
     instance::current();
-    client mongodb_client{uri{}};
+    client mongodb_client{uri{}, test_util::add_test_server_api()};
     if (!test_util::is_replica_set(mongodb_client)) {
         WARN("skip: change streams require replica set");
         return;
@@ -620,7 +636,7 @@ TEST_CASE("Watch a Collection", "[min36]") {
 
         SECTION("Can receive it") {
             auto it = *(x.begin());
-            REQUIRE(it["fullDocument"]["a"].get_utf8().value == stdx::string_view("b"));
+            REQUIRE(it["fullDocument"]["a"].get_string().value == stdx::string_view("b"));
         }
 
         SECTION("iterator equals itself") {
@@ -638,8 +654,8 @@ TEST_CASE("Watch a Collection", "[min36]") {
             auto it = x.begin();
             auto a = *it;
             auto b = *it;
-            REQUIRE(a["fullDocument"]["a"].get_utf8().value == stdx::string_view("b"));
-            REQUIRE(b["fullDocument"]["a"].get_utf8().value == stdx::string_view("b"));
+            REQUIRE(a["fullDocument"]["a"].get_string().value == stdx::string_view("b"));
+            REQUIRE(b["fullDocument"]["a"].get_string().value == stdx::string_view("b"));
         }
 
         SECTION("Calling .begin multiple times doesn't advance state") {
@@ -756,4 +772,4 @@ TEST_CASE("Watch a Collection", "[min36]") {
     }
 }
 
-}  // namepsace
+}  // namespace
