@@ -13,9 +13,6 @@
 // limitations under the License.
 
 #include "helpers.hpp"
-
-#include <mongocxx/config/private/prelude.hh>
-
 #include <bsoncxx/string/to_string.hpp>
 #include <bsoncxx/test_util/catch.hh>
 #include <mongocxx/client.hpp>
@@ -27,6 +24,8 @@
 #include <mongocxx/test_util/client_helpers.hh>
 #include <mongocxx/uri.hpp>
 
+#include <mongocxx/config/private/prelude.hh>
+
 namespace {
 using namespace mongocxx;
 
@@ -57,8 +56,7 @@ TEST_CASE("A client lists its databases with a filter applied", "[client]") {
     auto filter_doc = make_document(kvp("filter", make_document(kvp("name", "admin"))));
     auto filter_view = filter_doc.view();
 
-    auto client_list_databases = libmongoc::client_find_databases_with_opts.create_instance();
-    client_list_databases
+    client_find_databases_with_opts
         ->interpose([&](mongoc_client_t*, const bson_t* opts) {
             REQUIRE(opts);
             bsoncxx::document::view opts_view{bson_get_data(opts), opts->len};
@@ -68,17 +66,58 @@ TEST_CASE("A client lists its databases with a filter applied", "[client]") {
         })
         .forever();
 
-    client mongo_client{uri{}};
+    client mongo_client{uri{}, test_util::add_test_server_api()};
     mongo_client.list_databases(filter_view);
     REQUIRE(client_list_databases_called);
 }
 
+TEST_CASE("list databases passes authorizedDatabases option", "[client]") {
+    using bsoncxx::to_json;
+    using bsoncxx::builder::basic::kvp;
+    using bsoncxx::builder::basic::make_document;
+
+    MOCK_CLIENT
+
+    bool called = false;
+    stdx::optional<bsoncxx::document::value> opts_passed;
+
+    client_find_databases_with_opts->visit([&](mongoc_client_t*, const bson_t* opts) {
+        called = true;
+        if (opts) {
+            opts_passed =
+                bsoncxx::document::value{bsoncxx::document::view{bson_get_data(opts), opts->len}};
+        }
+        return nullptr;
+    });
+
+    mongocxx::client client{mongocxx::uri{}, test_util::add_test_server_api()};
+
+    SECTION("list_databases with no arguments") {
+        client.list_databases();
+        REQUIRE(called);
+    }
+
+    SECTION("list_databases with filter") {
+        bsoncxx::document::value opts = make_document(kvp("filter", 1));
+        client.list_databases(opts.view());
+        REQUIRE(called);
+        REQUIRE_BSON_MATCHES(*opts_passed, opts);
+    }
+
+    SECTION("list_databases with authorizedDatabases") {
+        bsoncxx::document::value opts = make_document(kvp("authorizedDatabases", true));
+        client.list_databases(opts.view());
+        REQUIRE(called);
+        REQUIRE_BSON_MATCHES(*opts_passed, opts);
+    }
+}
+
 TEST_CASE("A client constructed with a URI is truthy", "[client]") {
     MOCK_CLIENT
 
     instance::current();
 
-    client a{uri{}};
+    client a{uri{}, test_util::add_test_server_api()};
     REQUIRE(a);
 }
 
@@ -119,7 +158,7 @@ TEST_CASE("A client cleans up its underlying mongoc client on destruction", "[cl
     client_destroy->visit([&](mongoc_client_t*) { destroy_called = true; });
 
     {
-        client object{uri{}};
+        client object{uri{}, test_util::add_test_server_api()};
         REQUIRE(!destroy_called);
     }
 
@@ -131,7 +170,7 @@ TEST_CASE("A client supports move operations", "[client]") {
 
     instance::current();
 
-    client a{uri{}};
+    client a{uri{}, test_util::add_test_server_api()};
 
     bool called = false;
     client_new->visit([&](const mongoc_uri_t*) { called = true; });
@@ -148,7 +187,7 @@ TEST_CASE("A client has a settable Read Concern", "[client]") {
 
     instance::current();
 
-    client mongo_client{uri{}};
+    client mongo_client{uri{}, test_util::add_test_server_api()};
 
     auto client_set_rc_called = false;
     read_concern rc{};
@@ -172,7 +211,7 @@ TEST_CASE("A client's read preferences may be set and obtained", "[client]") {
 
     instance::current();
 
-    client mongo_client{uri{}};
+    client mongo_client{uri{}, test_util::add_test_server_api()};
     read_preference preference{};
     preference.mode(read_preference::read_mode::k_secondary_preferred);
 
@@ -211,7 +250,7 @@ TEST_CASE("A client may not change apm callbacks after they are set", "[client]"
 
     options::client client_opts;
     client_opts.apm_opts(apm_opts);
-    client mongo_client(uri{}, client_opts);
+    client mongo_client(uri{}, test_util::add_test_server_api(client_opts));
 
     apm_opts.on_command_started([&](const events::command_started_event& event) {
         INFO(event.command_name());
@@ -242,7 +281,7 @@ TEST_CASE("A client can delete apm options and the callbacks will still work pro
         client_opts.apm_opts(apm_opts);
     }  // destructor for apm_opts is called
 
-    client mongo_client(uri{}, client_opts);
+    client mongo_client(uri{}, test_util::add_test_server_api(client_opts));
 
     mongo_client["test"]["test_apm"].insert_one(
         bsoncxx::builder::basic::make_document(bsoncxx::builder::basic::kvp("x", 2)));
@@ -255,7 +294,7 @@ TEST_CASE("A client's write concern may be set and obtained", "[client]") {
 
     instance::current();
 
-    client mongo_client{uri{}};
+    client mongo_client{uri{}, test_util::add_test_server_api()};
     write_concern concern;
     concern.majority(std::chrono::milliseconds(100));
 
@@ -299,7 +338,7 @@ TEST_CASE("A client can be reset", "[client]") {
     bool reset_called = false;
     client_reset->interpose([&](const mongoc_client_t*) { reset_called = true; });
 
-    client mongo_client{uri{}};
+    client mongo_client{uri{}, test_util::add_test_server_api()};
     mongo_client.reset();
 
     REQUIRE(reset_called);
@@ -323,7 +362,7 @@ TEST_CASE("A client can create a named database object", "[client]") {
 
     stdx::string_view name("database");
 
-    client mongo_client{uri{}};
+    client mongo_client{uri{}, test_util::add_test_server_api()};
     database obtained_database = mongo_client[name];
     REQUIRE(obtained_database.name() == name);
 }
@@ -348,7 +387,7 @@ TEST_CASE("integration tests for client metadata handshake feature") {
             auto op_view = it.get_document().view();
 
             if (!op_view["appName"] ||
-                op_view["appName"].get_utf8().value != stdx::string_view(app_name)) {
+                op_view["appName"].get_string().value != stdx::string_view(app_name)) {
                 continue;
             }
 
@@ -364,14 +403,16 @@ TEST_CASE("integration tests for client metadata handshake feature") {
 
                 REQUIRE(metadata_view["application"]);
                 auto application = metadata_view["application"].get_document();
-                REQUIRE(application.view()["name"].get_utf8().value == stdx::string_view(app_name));
+                REQUIRE(application.view()["name"].get_string().value ==
+                        stdx::string_view(app_name));
 
                 REQUIRE(metadata_view["driver"]);
                 auto driver = metadata_view["driver"].get_document();
                 auto driver_view = driver.view();
-                REQUIRE(driver_view["name"].get_utf8().value ==
+                REQUIRE(driver_view["name"].get_string().value ==
                         stdx::string_view{"mongoc / mongocxx"});
-                auto version = bsoncxx::string::to_string(driver_view["version"].get_utf8().value);
+                auto version =
+                    bsoncxx::string::to_string(driver_view["version"].get_string().value);
                 REQUIRE(version.find(MONGOCXX_VERSION_STRING) != std::string::npos);
 
                 REQUIRE(metadata_view["os"]);
@@ -384,12 +425,12 @@ TEST_CASE("integration tests for client metadata handshake feature") {
     };
 
     SECTION("with client") {
-        mongocxx::client client{uri};
+        mongocxx::client client{uri, test_util::add_test_server_api()};
         run_test(client);
     }
 
     SECTION("with pool") {
-        mongocxx::pool pool{uri};
+        mongocxx::pool pool{uri, options::pool(test_util::add_test_server_api())};
         auto client = pool.acquire();
         run_test(*client);
     }
