@@ -13,15 +13,11 @@
 // limitations under the License.
 
 #include "helpers.hpp"
-
 #include <bsoncxx/builder/basic/document.hpp>
 #include <bsoncxx/builder/basic/kvp.hpp>
 #include <bsoncxx/test_util/catch.hh>
 #include <mongocxx/instance.hpp>
 #include <mongocxx/options/gridfs/upload.hpp>
-#include <mongocxx/read_concern.hpp>
-#include <mongocxx/read_preference.hpp>
-#include <mongocxx/write_concern.hpp>
 
 namespace {
 using namespace bsoncxx;
