@@ -12,8 +12,6 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "helpers.hpp"
-
 #include <bsoncxx/builder/basic/array.hpp>
 #include <bsoncxx/builder/basic/document.hpp>
 #include <bsoncxx/test_util/catch.hh>
@@ -25,8 +23,8 @@ namespace {
 using namespace bsoncxx;
 
 using bsoncxx::builder::basic::kvp;
-using bsoncxx::builder::basic::make_document;
 using bsoncxx::builder::basic::make_array;
+using bsoncxx::builder::basic::make_document;
 
 TEST_CASE("bulk_write result", "[bulk_write][result]") {
     mongocxx::instance::current();
@@ -85,4 +83,4 @@ TEST_CASE("bulk_write result inequals", "[bulk_write][result]") {
 
     REQUIRE(bw1 != bw2);
 }
-}
+}  // namespace
