@@ -13,17 +13,10 @@
 // limitations under the License.
 
 #include <fstream>
-#include <functional>
-#include <iostream>
-#include <map>
-#include <memory>
 #include <set>
-#include <sstream>
 #include <string>
 #include <vector>
 
-#include <mongocxx/test/spec/util.hh>
-
 #include <bsoncxx/builder/basic/array.hpp>
 #include <bsoncxx/builder/basic/document.hpp>
 #include <bsoncxx/document/value.hpp>
@@ -35,28 +28,16 @@
 #include <bsoncxx/test_util/catch.hh>
 #include <mongocxx/client.hpp>
 #include <mongocxx/collection.hpp>
-#include <mongocxx/cursor.hpp>
 #include <mongocxx/database.hpp>
 #include <mongocxx/exception/operation_exception.hpp>
-#include <mongocxx/instance.hpp>
-#include <mongocxx/options/aggregate.hpp>
-#include <mongocxx/options/count.hpp>
-#include <mongocxx/options/delete.hpp>
 #include <mongocxx/options/distinct.hpp>
-#include <mongocxx/options/find.hpp>
 #include <mongocxx/options/find_one_and_delete.hpp>
-#include <mongocxx/options/find_one_and_replace.hpp>
-#include <mongocxx/options/find_one_and_update.hpp>
-#include <mongocxx/options/find_one_common_options.hpp>
-#include <mongocxx/options/update.hpp>
 #include <mongocxx/pipeline.hpp>
 #include <mongocxx/result/delete.hpp>
-#include <mongocxx/result/insert_many.hpp>
 #include <mongocxx/result/insert_one.hpp>
-#include <mongocxx/result/replace_one.hpp>
-#include <mongocxx/result/update.hpp>
 #include <mongocxx/test/spec/monitoring.hh>
 #include <mongocxx/test/spec/operation.hh>
+#include <mongocxx/test/spec/util.hh>
 
 #include <mongocxx/config/private/prelude.hh>
 
@@ -73,7 +54,7 @@ using bsoncxx::builder::basic::make_document;
 using bsoncxx::stdx::optional;
 using bsoncxx::stdx::string_view;
 
-static const int kMaxIsMasterFailCommands = 7;
+static const int kMaxHelloFailCommands = 7;
 
 uint32_t error_code_from_name(string_view name) {
     if (name.compare("CannotSatisfyWriteConcern") == 0) {
@@ -94,6 +75,10 @@ uint32_t error_code_from_name(string_view name) {
         return 100;
     } else if (name.compare("OperationNotSupportedInTransaction") == 0) {
         return 263;
+    } else if (name.compare("APIStrictError") == 0) {
+        return 323;
+    } else if (name.compare("ChangeStreamFatalError") == 0) {
+        return 280;
     }
 
     return 0;
@@ -103,10 +88,15 @@ uint32_t error_code_from_name(string_view name) {
 bool should_skip_spec_test(const client& client, document::view test) {
     std::set<std::string> unsupported_tests = {
         "CreateIndex and dropIndex omits default write concern",
-        "MapReduce omits default write concern"};
+        "MapReduce omits default write concern",
+        "Deprecated count with empty collection",
+        "Deprecated count with collation",
+        "Deprecated count without a filter",
+        "Deprecated count with a filter",
+        "Deprecated count with skip and limit"};
 
     if (test["description"]) {
-        std::string description = std::string(test["description"].get_utf8().value);
+        std::string description = std::string(test["description"].get_string().value);
         if (unsupported_tests.find(description) != unsupported_tests.end()) {
             UNSCOPED_INFO("Test skipped - " << description << "\n"
                                             << "reason: unsupported in C++ driver");
@@ -115,18 +105,30 @@ bool should_skip_spec_test(const client& client, document::view test) {
     }
 
     if (test["skipReason"]) {
-        UNSCOPED_INFO("Test skipped - " << test["description"].get_utf8().value << "\n"
-                                        << "reason: "
-                                        << test["skipReason"].get_utf8().value);
+        UNSCOPED_INFO("Test skipped - " << test["description"].get_string().value << "\n"
+                                        << "reason: " << test["skipReason"].get_string().value);
         return true;
     }
 
-    std::string server_version = test_util::get_server_version(client);
+    std::string server_version;
+    try {
+        server_version = test_util::get_server_version(client);
+    } catch (const operation_exception& e) {
+        // Mongohouse does not support serverStatus, so if we get an error from
+        // serverStatus, exit this logic early and run the test.
+        std::string message = e.what();
+        if (message.find("command serverStatus is unsupported") != std::string::npos) {
+            return false;
+        }
+
+        throw e;
+    }
+
     std::string topology = test_util::get_topology(client);
 
     if (test["ignore_if_server_version_greater_than"]) {
         std::string max_server_version = bsoncxx::string::to_string(
-            test["ignore_if_server_version_greater_than"].get_utf8().value);
+            test["ignore_if_server_version_greater_than"].get_string().value);
         if (test_util::compare_versions(server_version, max_server_version) > 0) {
             return true;
         }
@@ -137,7 +139,7 @@ bool should_skip_spec_test(const client& client, document::view test) {
             auto topologies = requirements["topology"].get_array().value;
             bool found = false;
             for (auto&& el : topologies) {
-                if (std::string(el.get_utf8().value) == topology) {
+                if (std::string(el.get_string().value) == topology) {
                     found = true;
                     break;
                 }
@@ -149,7 +151,7 @@ bool should_skip_spec_test(const client& client, document::view test) {
 
         if (requirements["minServerVersion"]) {
             auto min_server_version =
-                string::to_string(requirements["minServerVersion"].get_utf8().value);
+                string::to_string(requirements["minServerVersion"].get_string().value);
             if (test_util::compare_versions(server_version, min_server_version) < 0) {
                 return true;
             }
@@ -157,7 +159,7 @@ bool should_skip_spec_test(const client& client, document::view test) {
 
         if (requirements["maxServerVersion"]) {
             auto max_server_version =
-                string::to_string(requirements["maxServerVersion"].get_utf8().value);
+                string::to_string(requirements["maxServerVersion"].get_string().value);
             if (test_util::compare_versions(server_version, max_server_version) > 0) {
                 return true;
             }
@@ -188,15 +190,15 @@ void configure_fail_point(const client& client, document::view test) {
 }
 
 void disable_fail_point(const client& client, stdx::string_view fail_point) {
-    /* Some transactions tests have a failCommand for "isMaster" repeat seven times. */
-    for (int i = 0; i < kMaxIsMasterFailCommands; i++) {
+    /* Some transactions tests have a failCommand for "hello" repeat seven times. */
+    for (int i = 0; i < kMaxHelloFailCommands; i++) {
         try {
             client["admin"].run_command(
                 make_document(kvp("configureFailPoint", fail_point), kvp("mode", "off")));
             break;
         } catch (const std::exception&) {
-            /* Tests that fail with isMaster also fail to disable the failpoint
-             * (since we run isMaster when opening the connection). Ignore those
+            /* Tests that fail with hello also fail to disable the failpoint
+             * (since we run hello when opening the connection). Ignore those
              * errors. */
             continue;
         }
@@ -217,10 +219,10 @@ void set_up_collection(const client& client,
     write_concern wc_majority;
     wc_majority.acknowledge_level(write_concern::level::k_majority);
 
-    auto db = client[test[database_name].get_utf8().value];
+    auto db = client[test[database_name].get_string().value];
     db.drop();
 
-    auto coll_name = test[collection_name].get_utf8().value;
+    auto coll_name = test[collection_name].get_string().value;
     auto coll = db[coll_name];
 
     coll.drop(wc_majority);
@@ -336,17 +338,15 @@ void run_operation_check_result(document::view op, make_op_runner_fn make_op_run
     // matches the error string."
     if (op["result"]["errorContains"]) {
         REQUIRE(exception);
-        INFO("expected error message " << op["result"]["errorContains"].get_utf8().value);
+        INFO("expected error message " << op["result"]["errorContains"].get_string().value);
         INFO("got error message" << error_msg);
         // Do a case insensitive check.
         auto error_contains =
-            test_util::tolowercase(op["result"]["errorContains"].get_utf8().value);
+            test_util::tolowercase(op["result"]["errorContains"].get_string().value);
         REQUIRE(test_util::tolowercase(error_msg).find(error_contains) < error_msg.length());
-    } else {
-        if (exception) {
-            FAIL("operation " << bsoncxx::to_json(op) << " threw an unexpected exception: "
-                              << exception->what());
-        }
+    } else if (exception) {
+        CAPTURE(server_error);
+        FAIL("unexpected exception: " << error_msg);
     }
 
     // "If the result document has an 'errorCodeName' field, verify that the method threw a
@@ -354,7 +354,7 @@ void run_operation_check_result(document::view op, make_op_runner_fn make_op_run
     // 'errorCodeName' field matches the 'codeName' in the server error response."
     if (op["result"]["errorCodeName"]) {
         REQUIRE(op_exception);
-        uint32_t expected = error_code_from_name(op["result"]["errorCodeName"].get_utf8().value);
+        uint32_t expected = error_code_from_name(op["result"]["errorCodeName"].get_string().value);
         REQUIRE(op_exception->code().value() == static_cast<int>(expected));
     }
 
@@ -363,7 +363,7 @@ void run_operation_check_result(document::view op, make_op_runner_fn make_op_run
     if (op["result"]["errorLabelsContain"]) {
         REQUIRE(op_exception);
         for (auto&& label_el : op["result"]["errorLabelsContain"].get_array().value) {
-            auto label = label_el.get_utf8().value;
+            auto label = label_el.get_string().value;
             REQUIRE(op_exception->has_error_label(label));
         }
     }
@@ -373,7 +373,7 @@ void run_operation_check_result(document::view op, make_op_runner_fn make_op_run
     if (op["result"]["errorLabelsOmit"]) {
         REQUIRE(op_exception);
         for (auto&& label_el : op["result"]["errorLabelsOmit"].get_array().value) {
-            auto label = label_el.get_utf8().value;
+            auto label = label_el.get_string().value;
             REQUIRE(!op_exception->has_error_label(label));
         }
     }
@@ -412,13 +412,13 @@ uri get_uri(document::view test) {
         }
         if (test["clientOptions"]["readConcernLevel"]) {
             add_opt("readConcernLevel=" +
-                    std::string(test["clientOptions"]["readConcernLevel"].get_utf8().value));
+                    std::string(test["clientOptions"]["readConcernLevel"].get_string().value));
         }
         if (test["clientOptions"]["w"]) {
             if (test["clientOptions"]["w"].type() == type::k_int32) {
                 add_opt("w=" + std::to_string(test["clientOptions"]["w"].get_int32().value));
             } else {
-                add_opt("w=" + string::to_string(test["clientOptions"]["w"].get_utf8().value));
+                add_opt("w=" + string::to_string(test["clientOptions"]["w"].get_string().value));
             }
         }
         if (test["clientOptions"]["heartbeatFrequencyMS"]) {
@@ -428,7 +428,7 @@ uri get_uri(document::view test) {
         }
         if (test["clientOptions"]["readPreference"]) {
             add_opt("readPreference=" +
-                    string::to_string(test["clientOptions"]["readPreference"].get_utf8().value));
+                    string::to_string(test["clientOptions"]["readPreference"].get_string().value));
         }
     }
     return uri{uri_string};
@@ -437,6 +437,7 @@ uri get_uri(document::view test) {
 void run_tests_in_suite(std::string ev, test_runner cb, std::set<std::string> unsupported_tests) {
     char* tests_path = std::getenv(ev.c_str());
     INFO("checking for path from environment variable: " << ev);
+    INFO("test path is: " << tests_path);
     REQUIRE(tests_path);
 
     std::string path{tests_path};
@@ -449,11 +450,13 @@ void run_tests_in_suite(std::string ev, test_runner cb, std::set<std::string> un
 
     std::string test_file;
     while (std::getline(test_files, test_file)) {
-        if (unsupported_tests.find(test_file) != unsupported_tests.end()) {
-            WARN("Skipping unsupported test file: " << test_file);
-            continue;
+        SECTION(test_file) {
+            if (unsupported_tests.find(test_file) != unsupported_tests.end()) {
+                WARN("Skipping unsupported test file: " << test_file);
+            } else {
+                cb(path + "/" + test_file);
+            }
         }
-        cb(path + "/" + test_file);
     }
 }
 
@@ -464,7 +467,7 @@ void run_tests_in_suite(std::string ev, test_runner cb) {
 
 void test_setup(document::view test, document::view test_spec) {
     // Step 1. "clean up any open transactions from previous test failures"
-    client client{uri{}};
+    client client{uri{}, test_util::add_test_server_api()};
     try {
         client["admin"].run_command(make_document(kvp("killAllSessions", make_array())));
     } catch (const operation_exception& e) {
@@ -526,9 +529,9 @@ void run_transaction_operations(document::view test,
         auto operation = op.get_document().value;
 
         // Handle with_transaction separately.
-        if (operation["name"].get_utf8().value.compare("withTransaction") == 0) {
+        if (operation["name"].get_string().value.compare("withTransaction") == 0) {
             auto session = [&]() {
-                if (operation["object"].get_utf8().value.compare("session0") == 0) {
+                if (operation["object"].get_string().value.compare("session0") == 0) {
                     return session0;
                 } else {
                     return session1;
@@ -582,7 +585,7 @@ void run_transaction_operations(document::view test,
             REQUIRE(exception);
             // Do a case insensitive check.
             auto error_contains =
-                test_util::tolowercase(op["result"]["errorContains"].get_utf8().value);
+                test_util::tolowercase(op["result"]["errorContains"].get_string().value);
             REQUIRE(test_util::tolowercase(error_msg).find(error_contains) < error_msg.length());
         }
 
@@ -593,7 +596,7 @@ void run_transaction_operations(document::view test,
             REQUIRE(exception);
             REQUIRE(server_error);
             uint32_t expected =
-                error_code_from_name(op["result"]["errorCodeName"].get_utf8().value);
+                error_code_from_name(op["result"]["errorCodeName"].get_string().value);
             REQUIRE(exception->code().value() == static_cast<int>(expected));
         }
 
@@ -602,7 +605,7 @@ void run_transaction_operations(document::view test,
         if (op["result"]["errorLabelsContain"]) {
             REQUIRE(exception);
             for (auto&& label_el : op["result"]["errorLabelsContain"].get_array().value) {
-                auto label = label_el.get_utf8().value;
+                auto label = label_el.get_string().value;
                 if (!exception->has_error_label(label)) {
                     FAIL("Expected exception to contain the label '" << label
                                                                      << "' but it did not.\n");
@@ -615,7 +618,7 @@ void run_transaction_operations(document::view test,
         if (op["result"]["errorLabelsOmit"]) {
             REQUIRE(exception);
             for (auto&& label_el : op["result"]["errorLabelsOmit"].get_array().value) {
-                auto label = label_el.get_utf8().value;
+                auto label = label_el.get_string().value;
                 if (exception->has_error_label(label)) {
                     FAIL("Expected exception to NOT contain the label '" << label
                                                                          << "' but it did.\n");
@@ -647,21 +650,22 @@ void run_transactions_tests_in_file(const std::string& test_path) {
     auto test_spec = test_util::parse_test_file(test_path);
     REQUIRE(test_spec);
     auto test_spec_view = test_spec->view();
-    auto db_name = test_spec_view["database_name"].get_utf8().value;
-    auto coll_name = test_spec_view["collection_name"].get_utf8().value;
+    auto db_name = test_spec_view["database_name"].get_string().value;
+    auto coll_name = test_spec_view["collection_name"].get_string().value;
     auto tests = test_spec_view["tests"].get_array().value;
 
     /* we may not have a supported topology */
-    if (should_skip_spec_test(client{uri{}}, test_spec_view)) {
+    if (should_skip_spec_test(client{uri{}, test_util::add_test_server_api()}, test_spec_view)) {
         WARN("File skipped - " + test_path);
         return;
     }
 
     for (auto&& test : tests) {
         bool fail_point_enabled = (bool)test["failPoint"];
-        auto description = test["description"].get_utf8().value;
+        auto description = test["description"].get_string().value;
         INFO("Test description: " << description);
-        if (should_skip_spec_test(client{uri{}}, test.get_document().value)) {
+        if (should_skip_spec_test(client{uri{}, test_util::add_test_server_api()},
+                                  test.get_document().value)) {
             continue;
         }
 
@@ -672,6 +676,7 @@ void run_transactions_tests_in_file(const std::string& test_path) {
         options::client client_opts;
         apm_checker apm_checker;
         client_opts.apm_opts(apm_checker.get_apm_opts(true /* command_started_events_only */));
+        client_opts = test_util::add_test_server_api(client_opts);
         client client;
         if (test["useMultipleMongoses"]) {
             client = {uri{"mongodb://localhost:27017,localhost:27018"}, client_opts};
@@ -725,29 +730,29 @@ void run_transactions_tests_in_file(const std::string& test_path) {
         }
 
         // Step 11. Compare APM events.
-        test_util::match_visitor visitor = [&](
-            bsoncxx::stdx::string_view key,
-            bsoncxx::stdx::optional<bsoncxx::types::bson_value::view> main,
-            bsoncxx::types::bson_value::view pattern) {
-            if (key.compare("lsid") == 0) {
-                REQUIRE(pattern.type() == type::k_utf8);
-                REQUIRE(main);
-                REQUIRE(main->type() == type::k_document);
-                auto session_name = pattern.get_utf8().value;
-                if (session_name.compare("session0") == 0) {
-                    REQUIRE(test_util::matches(session_lsid0, main->get_document().value));
-                } else {
-                    REQUIRE(test_util::matches(session_lsid1, main->get_document().value));
+        test_util::match_visitor visitor =
+            [&](bsoncxx::stdx::string_view key,
+                bsoncxx::stdx::optional<bsoncxx::types::bson_value::view> main,
+                bsoncxx::types::bson_value::view pattern) {
+                if (key.compare("lsid") == 0) {
+                    REQUIRE(pattern.type() == type::k_string);
+                    REQUIRE(main);
+                    REQUIRE(main->type() == type::k_document);
+                    auto session_name = pattern.get_string().value;
+                    if (session_name.compare("session0") == 0) {
+                        REQUIRE(test_util::matches(session_lsid0, main->get_document().value));
+                    } else {
+                        REQUIRE(test_util::matches(session_lsid1, main->get_document().value));
+                    }
+                    return test_util::match_action::k_skip;
+                } else if (pattern.type() == type::k_null) {
+                    if (main) {
+                        return test_util::match_action::k_not_equal;
+                    }
+                    return test_util::match_action::k_skip;
                 }
                 return test_util::match_action::k_skip;
-            } else if (pattern.type() == type::k_null) {
-                if (main) {
-                    return test_util::match_action::k_not_equal;
-                }
-                return test_util::match_action::k_skip;
-            }
-            return test_util::match_action::k_skip;
-        };
+            };
 
         if (test["expectations"]) {
             apm_checker.compare(test["expectations"].get_array().value, false, visitor);
@@ -765,7 +770,7 @@ void run_transactions_tests_in_file(const std::string& test_path) {
         if (test["outcome"] && test["outcome"]["collection"]) {
             auto outcome_coll_name = coll_name;
             if (test["outcome"]["collection"]["name"]) {
-                outcome_coll_name = test["outcome"]["collection"]["name"].get_utf8().value;
+                outcome_coll_name = test["outcome"]["collection"]["name"].get_string().value;
             }
             auto coll = client[db_name][outcome_coll_name];
             test_util::check_outcome_collection(&coll,
@@ -774,6 +779,137 @@ void run_transactions_tests_in_file(const std::string& test_path) {
     }
 }
 
+void run_crud_tests_in_file(const std::string& test_path, uri test_uri) {
+    using bsoncxx::builder::basic::kvp;
+    using bsoncxx::builder::basic::make_document;
+
+    using bsoncxx::string::to_string;
+
+    INFO("Test path: " << test_path);
+    optional<document::value> test_spec = test_util::parse_test_file(test_path);
+    REQUIRE(test_spec);
+
+    options::client client_opts;
+    apm_checker apm_checker;
+    client_opts.apm_opts(apm_checker.get_apm_opts(true /* command_started_events_only */));
+    client_opts = test_util::add_test_server_api(client_opts);
+    client client{std::move(test_uri), client_opts};
+
+    document::view test_spec_view = test_spec->view();
+    if (should_skip_spec_test(client, test_spec_view)) {
+        return;
+    }
+
+    for (auto&& test : test_spec_view["tests"].get_array().value) {
+        auto description = test["description"].get_string().value;
+        SECTION(to_string(description)) {
+            if (should_skip_spec_test(client, test.get_document())) {
+                continue;
+            }
+
+            auto get_value_or_default = [&](std::string key, std::string default_str) {
+                if (test_spec_view[key]) {
+                    return to_string(test_spec_view[key].get_string().value);
+                }
+                return default_str;
+            };
+
+            auto database_name = get_value_or_default("database_name", "crud_test");
+            auto collection_name = get_value_or_default("collection_name", "test");
+
+            auto database = client[database_name];
+            auto collection = database[collection_name];
+
+            if (test_spec_view["data"]) {
+                initialize_collection(&collection, test_spec_view["data"].get_array().value);
+            }
+
+            operation_runner op_runner{&database, &collection, nullptr, nullptr, &client};
+
+            std::string outcome_collection_name = collection_name;
+            if (test["outcome"] && test["outcome"]["collection"]["name"]) {
+                outcome_collection_name =
+                    to_string(test["outcome"]["collection"]["name"].get_string().value);
+                auto outcome_collection = database[outcome_collection_name];
+                initialize_collection(&outcome_collection, array::view{});
+            }
+
+            configure_fail_point(client, test.get_document().value);
+
+            apm_checker.clear();
+            auto perform_op =
+                [&database, &op_runner, &test, &outcome_collection_name](document::view operation) {
+                    optional<document::value> actual_outcome_value;
+                    INFO("Operation: " << bsoncxx::to_json(operation));
+                    try {
+                        actual_outcome_value = op_runner.run(operation);
+                    } catch (const mongocxx::operation_exception& e) {
+                        REQUIRE([&operation, &test, &e]() {
+                            if (operation["error"]) { /* v2 tests expect tests[i].operation.error */
+                                return operation["error"].get_bool().value;
+                            } else if (test["outcome"] && test["outcome"]["error"]) {
+                                /* v1 tests expect tests[i].outcome.error (but some tests may
+                                 have "outcome" without a nested "error") */
+                                return test["outcome"]["error"].get_bool().value;
+                            } else {
+                                WARN("Caught operation exception: " << e.what());
+                                return false;
+                            }
+                        }());
+                        return; /* do not check results if error is expected */
+                    } catch (const std::exception& e) {
+                        WARN("Caught exception: " << e.what());
+                    } catch (...) {
+                        WARN("Caught unknown exception");
+                    }
+
+                    if (test["outcome"]) {
+                        if (test["outcome"]["collection"]) {
+                            auto outcome_collection = database[outcome_collection_name];
+                            test_util::check_outcome_collection(
+                                &outcome_collection,
+                                test["outcome"]["collection"].get_document().value);
+                        }
+
+                        if (test["outcome"]["result"]) {
+                            // wrap the result, since it might not be a document.
+                            bsoncxx::document::view actual_outcome = actual_outcome_value->view();
+                            auto actual_result_wrapped =
+                                make_document(kvp("result", actual_outcome["result"].get_value()));
+                            auto expected_result_wrapped =
+                                make_document(kvp("result", test["outcome"]["result"].get_value()));
+                            REQUIRE_BSON_MATCHES(actual_result_wrapped, expected_result_wrapped);
+                        }
+                    }
+                };
+
+            if (test["operations"]) {
+                /* v2 tests expect a tests[i].operations array */
+                for (auto&& operation : test["operations"].get_array().value) {
+                    perform_op(operation.get_document().value);
+                }
+            } else if (test["operation"]) {
+                /* v1 tests expect a single document, tests[i].operation */
+                perform_op(test["operation"].get_document().value);
+            }
+
+            if (test["expectations"]) {
+                /* some tests use empty documents, instead of arrays */
+                if (test["expectations"].type() == type::k_array) {
+                    apm_checker.compare(test["expectations"].get_array().value, true);
+                } else {
+                    REQUIRE(test["expectations"].get_document().view().empty());
+                }
+            }
+
+            if (test["failPoint"]) {
+                disable_fail_point(client,
+                                   test["failPoint"]["configureFailPoint"].get_string().value);
+            }
+        }
+    }
+}
+
 }  // namespace spec
 MONGOCXX_INLINE_NAMESPACE_END
 }  // namespace mongocxx
