@@ -12,11 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include <mongocxx/test_util/client_helpers.hh>
-
 #include <algorithm>
 #include <fstream>
-#include <functional>
 #include <sstream>
 #include <string>
 #include <utility>
@@ -32,12 +29,13 @@
 #include <bsoncxx/stdx/string_view.hpp>
 #include <bsoncxx/string/to_string.hpp>
 #include <bsoncxx/types.hpp>
+#include <bsoncxx/types/bson_value/view_or_value.hpp>
 #include <mongocxx/client.hpp>
 #include <mongocxx/exception/error_code.hpp>
 #include <mongocxx/exception/logic_error.hpp>
 #include <mongocxx/exception/operation_exception.hpp>
 #include <mongocxx/private/libmongoc.hh>
-#include <mongocxx/stdx.hpp>
+#include <mongocxx/test_util/client_helpers.hh>
 #include <third_party/catch/include/catch.hpp>
 
 #include <mongocxx/config/private/prelude.hh>
@@ -53,6 +51,24 @@ using namespace bsoncxx;
 using bsoncxx::builder::basic::kvp;
 using bsoncxx::builder::basic::make_document;
 
+namespace {
+// These frequently used network calls are cached to avoid bottlenecks during tests.
+document::value get_is_master(const client& client) {
+    static auto reply = client["admin"].run_command(make_document(kvp("isMaster", 1)));
+    return reply;
+}
+
+document::value get_server_status(const client& client) {
+    static auto status = client["admin"].run_command(make_document(kvp("serverStatus", 1)));
+    return status;
+}
+
+stdx::optional<document::value> get_shards(const client& client) {
+    static auto shards = client["config"]["shards"].find_one({});
+    return (shards) ? shards.value() : stdx::optional<document::value>{};
+}
+}  // namespace
+
 std::vector<std::int32_t> parse_version(std::string version) {
     std::vector<std::int32_t> elements;
     std::stringstream ss{version};
@@ -183,8 +199,18 @@ std::basic_string<std::uint8_t> convert_hex_string_to_bytes(stdx::string_view he
     return bytes;
 }
 
+options::client add_test_server_api(options::client opts) {
+    auto api_version = std::getenv("MONGODB_API_VERSION");
+    // Check if MONGODB_API_VERSION is set and not empty.
+    if (api_version && std::string(api_version).length() > 0) {
+        auto version = options::server_api::version_from_string(api_version);
+        opts.server_api_opts(options::server_api(version));
+    }
+    return opts;
+}
+
 std::int32_t get_max_wire_version(const client& client) {
-    auto reply = client["admin"].run_command(make_document(kvp("isMaster", 1)));
+    auto reply = get_is_master(client);
     auto max_wire_version = reply.view()["maxWireVersion"];
     if (!max_wire_version) {
         // If wire version is not available (i.e. server version too old), it is assumed to be
@@ -198,38 +224,65 @@ std::int32_t get_max_wire_version(const client& client) {
 }
 
 std::string get_server_version(const client& client) {
-    bsoncxx::builder::basic::document server_status{};
-    server_status.append(bsoncxx::builder::basic::kvp("serverStatus", 1));
-    bsoncxx::document::value output = client["test"].run_command(server_status.extract());
+    auto output = get_server_status(client);
+    return bsoncxx::string::to_string(output.view()["version"].get_string().value);
+}
 
-    return bsoncxx::string::to_string(output.view()["version"].get_utf8().value);
+document::value get_server_params(const client& client) {
+    auto reply = client["admin"].run_command(make_document(kvp("getParameter", "*")));
+    return reply;
 }
 
 std::string replica_set_name(const client& client) {
-    auto reply = client["admin"].run_command(make_document(kvp("isMaster", 1)));
+    auto reply = get_is_master(client);
     auto name = reply.view()["setName"];
     if (name) {
-        return bsoncxx::string::to_string(name.get_utf8().value);
+        return bsoncxx::string::to_string(name.get_string().value);
     }
 
     return "";
 }
 
 bool is_replica_set(const client& client) {
-    auto reply = client["admin"].run_command(make_document(kvp("isMaster", 1)));
+    auto reply = get_is_master(client);
     return static_cast<bool>(reply.view()["setName"]);
 }
 
-std::string get_topology(const client& client) {
-    auto reply = client["admin"].run_command(make_document(kvp("isMaster", 1)));
-    if (reply.view()["setName"]) {
-        return "replicaset";
-    } else if (reply.view()["msg"] &&
-               std::string(reply.view()["msg"].get_utf8().value) == "isdbgrid") {
-        return "sharded";
-    } else {
-        return "single";
+bool is_sharded_cluster(const client& client) {
+    const auto reply = get_is_master(client);
+    const auto msg = reply.view()["msg"];
+
+    if (!msg) {
+        return false;
     }
+
+    return msg.get_string().value.compare("isdbgrid") == 0;
+}
+
+std::string get_hosts(const client& client) {
+    auto shards = get_shards(client);
+    if (shards)
+        return string::to_string(shards->view()["host"].get_string().value);
+    return "";
+}
+
+std::string get_topology(const client& client) {
+    if (is_replica_set(client))
+        return "replicaset";
+
+    // from: https://docs.mongodb.com/manual/reference/config-database/#config.shards
+    // If the shard is a replica set, the host field displays the name of the replica set, then a
+    // slash, then a comma-separated list of the hostnames of each member of the replica set, as in
+    // the following example:
+    //      { ... , "host" : "shard0001/localhost:27018,localhost:27019,localhost:27020", ... }
+    auto host = get_hosts(client);
+    if (!host.empty()) {
+        if (std::find(std::begin(host), std::end(host), '/') != std::end(host))
+            return "sharded-replicaset";
+        return "sharded";
+    }
+
+    return "single";
 }
 
 stdx::optional<bsoncxx::document::value> parse_test_file(std::string path) {
@@ -276,7 +329,7 @@ bool is_numeric(types::bson_value::view value) {
 
 stdx::optional<type> is_type_operator(types::bson_value::view value) {
     if (value.type() == type::k_document && value.get_document().value["$$type"]) {
-        auto t = value.get_document().value["$$type"].get_utf8().value;
+        auto t = value.get_document().value["$$type"].get_string().value;
         if (t.compare("binData") == 0) {
             return {type::k_binary};
         } else if (t.compare("long") == 0) {
@@ -291,7 +344,7 @@ bool matches(types::bson_value::view main,
              types::bson_value::view pattern,
              match_visitor visitor_fn) {
     if (auto t = is_type_operator(pattern)) {
-        return t == main.type() ? true : false;
+        return t == main.type();
     }
 
     if (is_numeric(pattern) && as_double(pattern) == 42) {
@@ -305,7 +358,7 @@ bool matches(types::bson_value::view main,
 
     if (main.type() == type::k_document) {
         // the value '42' acts as placeholders for "any value"
-        if (pattern.type() == type::k_utf8 && 0 == pattern.get_utf8().value.compare("42")) {
+        if (pattern.type() == type::k_string && 0 == pattern.get_string().value.compare("42")) {
             return true;
         }
 
@@ -356,7 +409,7 @@ bool matches(types::bson_value::view main,
         array::view main_array = main.get_array().value;
         array::view pattern_array = pattern.get_array().value;
 
-        if (main_array.length() < pattern_array.length()) {
+        if (size(main_array) < size(pattern_array)) {
             return false;
         }
 
@@ -412,7 +465,7 @@ void check_outcome_collection(mongocxx::collection* coll, bsoncxx::document::vie
 }
 
 bool server_has_sessions(const client& conn) {
-    auto result = conn["admin"].run_command(make_document(kvp("isMaster", 1)));
+    auto result = get_is_master(conn);
     auto result_view = result.view();
 
     if (result_view["logicalSessionTimeoutMinutes"]) {
@@ -429,21 +482,49 @@ bool should_run_client_side_encryption_test(void) {
     return false;
 #endif
 
-    auto access_key = std::getenv("MONGOCXX_TEST_AWS_SECRET_ACCESS_KEY");
-    auto key_id = std::getenv("MONGOCXX_TEST_AWS_ACCESS_KEY_ID");
+    std::vector<const char*> vars{
+        "MONGOCXX_TEST_AWS_SECRET_ACCESS_KEY",
+        "MONGOCXX_TEST_AWS_ACCESS_KEY_ID",
+        "MONGOCXX_TEST_AZURE_TENANT_ID",
+        "MONGOCXX_TEST_AZURE_CLIENT_ID",
+        "MONGOCXX_TEST_AZURE_CLIENT_SECRET",
+        "MONGOCXX_TEST_CSFLE_TLS_CA_FILE",
+        "MONGOCXX_TEST_CSFLE_TLS_CERTIFICATE_KEY_FILE",
+        "MONGOCXX_TEST_GCP_EMAIL",
+        "MONGOCXX_TEST_GCP_PRIVATEKEY",
+    };
+
+    std::ostringstream os;
+    os << "Please set environment variables to enable client side encryption tests:\n";
+    std::copy(std::begin(vars), std::end(vars), std::ostream_iterator<const char*>(os, "\n"));
+
+    if (std::none_of(std::begin(vars), std::end(vars), std::getenv)) {
+        os << "Skipping client side encryption tests.\n";
+
+        WARN(os.str());
+
+        return false;
+    }
+
+    if (!std::all_of(std::begin(vars), std::end(vars), std::getenv)) {
+        os << "Failing client side encryption tests (some environment variables were not set).\n";
+
+        FAIL(os.str());
 
-    if (!access_key || !key_id) {
-        WARN(
-            "Skipping tests. Please set environment variables to enable client side encryption "
-            "tests:\n"
-            "\tMONGOCXX_TEST_AWS_SECRET_ACCESS_KEY\n"
-            "\tMONGOCXX_TEST_AWS_ACCESS_KEY_ID\n\n");
         return false;
     }
 
     return true;
 }
 
+std::string getenv_or_fail(const std::string env_name) {
+    auto val = std::getenv(env_name.c_str());
+    if (!val) {
+        FAIL("Please set the environment variable: " << env_name);
+    }
+    return val;
+}
+
 }  // namespace test_util
 MONGOCXX_INLINE_NAMESPACE_END
 }  // namespace mongocxx
